<?php

namespace Drupal\Tests\message_digest_ui\Kernel;

use Drupal\flag\Entity\Flagging;
use Drupal\Tests\message_subscribe_email\Kernel\MessageSubscribeEmailTestBase;

/**
 * Test message digest actions.
 *
 * @group message_digest
 */
class DigestActionsTest extends MessageSubscribeEmailTestBase {

  /**
   * The action manager service.
   *
   * @var \Drupal\Core\Action\ActionManager
   */
  protected $actionManager;

  /**
   * Action config storage.
   *
   * @var \Drupal\Core\Config\Entity\ConfigEntityStorageInterface
   */
  protected $actionStorage;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'message_digest',
    'message_digest_ui',
    'options',
    'field',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    $this->actionManager = $this->container->get('plugin.manager.action');
    $this->actionStorage = $this->container->get('entity_type.manager')->getStorage('action');

    $this->actionManager->clearCachedDefinitions();
    $this->installConfig(['message_digest', 'message_digest_ui']);

    // Fake user 2 login.
    $this->container->get('account_switcher')->switchTo($this->users[2]);

    // Verify our flagging field is installed.
    assert($this->container->get('entity_type.manager')->getStorage('field_storage_config')->load('flagging.message_digest'));
  }

  /**
   * Test actions.
   *
   * @dataProvider providerTestActions
   */
  public function testActions($action_id, $interval_plugin) {
    /** @var \Drupal\system\Entity\Action $action */
    $action = $this->actionStorage->load($action_id);
    $plugin = $action->getPlugin();
    $flag = $this->flagService->getFlagById('email_node');

    // Access should be false when not flagged.
    $this->assertFalse($plugin->access($this->nodes[1], $this->users[2]));
    $access = $plugin->access($this->nodes[1], $this->users[2], TRUE);
    $this->assertFalse($access->isAllowed());

    // Flag node 1 on behalf of user 2.
    $this->flagService->flag($flag, $this->nodes[1], $this->users[2]);

    if ($interval_plugin === '0') {
      // If testing the immediate interval, set digest to something else.
      $flagging = $this->flagService->getFlagging($flag, $this->nodes[1], $this->users[2]);
      $flagging->message_digest = 'message_digest:daily';
      $flagging->save();
    }

    // Access should be TRUE.
    $this->assertTrue($plugin->access($this->nodes[1], $this->users[2]));
    $access = $plugin->access($this->nodes[1], $this->users[2], TRUE);
    $this->assertTrue($access->isAllowed());

    // Trigger plugin.
    $plugin->execute($this->nodes[1]);

    $flagging = $this->flagService->getFlagging($flag, $this->nodes[1], $this->users[2]);
    $this->assertEquals($interval_plugin, $flagging->message_digest->value);
  }

  /**
   * Data provider for testActions.
   */
  public function providerTestActions() {
    $cases = [];

    foreach (['immediate', 'daily', 'weekly'] as $interval) {
      $cases[] = ['message_digest_interval.email_node.' . $interval, $interval === 'immediate' ? '0' : 'message_digest:' . $interval, 'email_node'];
    }

    return $cases;
  }

}
